import fastapi
from fastapi import Depends, HTTPException, Response
from auth import auth
import logging
import conf
from starlette_context import context
from fastapi.responses import StreamingResponse
from bundles import bundle
import correlation
from datetime import date
from pydantic import BaseModel

from starlette.status import HTTP_400_BAD_REQUEST, HTTP_424_FAILED_DEPENDENCY

router = fastapi.APIRouter()


class Detail(BaseModel):
    detail: str


_logger = logging.getLogger(__name__)

description = f"""
## Experimental Backup API.

Allows downloading the bundle for a data partition.

- Bundle filename will be in the form bundle-`data-partition-id`-`date`.tar.gz.
- Date will be in YYYY-MM-DD format.

### Permission required to use this API:

    {conf.ADMIN_PERMISSION}
"""


@router.get(
    "/backup",
    responses={
        HTTP_400_BAD_REQUEST: {"model": Detail},
        HTTP_424_FAILED_DEPENDENCY: {"model": Detail},
    },
    description=description,
)
def backup(
    response: Response, auth_data: auth.Auth = Depends(auth.require_authorized_admin)
):
    """
    Experimental Backup API.

    Allows downloading the bundle for a data partition.

    Bundle filename will be in the form bundle-`data partition`-`date`.tar.gz
    """
    logging.setLogRecordFactory(
        correlation.set_correlation_id(context["correlation_id"])
    )
    logger = logging.getLogger(__name__)

    response.headers["X-Correlation-ID"] = context["correlation_id"]

    today = date.today()
    filename = f"bundle-{auth_data.data_partition_id}-{today}.tar.gz"

    if conf.CLOUD_PROVIDER == conf.LOCAL:
        detail = f"Backup of bundle for partition '{auth_data.data_partition_id}' not supported!"
        raise HTTPException(status_code=HTTP_424_FAILED_DEPENDENCY, detail=detail)

    data = bundle.get_bundle(data_partition=auth_data.data_partition_id)
    headers = {"Content-Disposition": f"attachment; filename={filename}"}
    if data:
        return StreamingResponse(data, headers=headers)
    else:
        detail = (
            f"Backup of bundle for partition '{auth_data.data_partition_id}' failed!"
        )
        logger.error(detail)
        raise HTTPException(status_code=HTTP_424_FAILED_DEPENDENCY, detail=detail)
