from fastapi import Depends, Request, Header, HTTPException
from fastapi.security import HTTPAuthorizationCredentials, HTTPBearer
from starlette.authentication import AuthCredentials
from requests.structures import CaseInsensitiveDict
import logging
import conf
from starlette_context import context
from entitlement import EntitlementService
import re
import correlation

from starlette.status import (
    HTTP_400_BAD_REQUEST,
    HTTP_401_UNAUTHORIZED,
    HTTP_403_FORBIDDEN,
)

_logger = logging.getLogger(__name__)
logger = correlation.DefaultExtrasAdapter(_logger, {"correlation_id": "None"})

security = HTTPBearer()


async def require_token(
    request: Request, credentials: HTTPAuthorizationCredentials = Depends(security)
):
    token = credentials.credentials
    request.scope["auth"] = AuthCredentials(["authenticated"])
    return token


async def require_data_partition_id(
    data_partition_id: str = Header(
        default=None,
        title="data partition id",
        description="identifier of the data partition to query",
        min_length=1,
    ),
):
    if data_partition_id is None:
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST,
            detail="Bad Request. Missing data_partition_id in header",
            headers={"WWW-Authenticate": "Bearer"},
        )
    return data_partition_id


class Auth:
    def __init__(
        self,
        access_token,
        data_partition_id=None,
        x_user_id=None,
        authenticated=False,
        entitle_service=EntitlementService,
    ):
        # def __init__(self, access_token, data_partition_id = None, x_user_id = None, authenticated = False):
        self.access_token = access_token
        self.data_partition_id = data_partition_id
        self.x_user_id = x_user_id
        self.authenticated = authenticated
        self.groups = []
        self.user_id = None
        self.member_email = None
        self.entitle_service = entitle_service

    def __repr__(self):
        return f"AUTH access_token: {self.access_token}\ndata_partition_id: {self.data_partition_id}\nx_user_id: {self.x_user_id}\ngroups: {self.groups}"


async def require_authorized_user(
    credentials: HTTPAuthorizationCredentials = Depends(security),
    data_partition_id: str = Header(
        default=None,
        description="identifier of the data partition to query",
        min_length=1,
    ),
    x_user_id: str = Header(
        default=None, description="identifier the user in the query", min_length=1
    ),
):
    token = credentials.credentials.strip()

    # handle junk in data_partition_id
    data_partition_id = str(data_partition_id)
    if not re.match(
        pattern=conf.ALLOW_DATA_PARTITION_ID_PATTERN, string=data_partition_id
    ):
        raise HTTPException(
            status_code=HTTP_400_BAD_REQUEST, detail="Unsupported data-partition-id"
        )

    auth_data = Auth(
        access_token=token, data_partition_id=data_partition_id, x_user_id=x_user_id
    )
    headers = CaseInsensitiveDict()
    headers["Authorization"] = "Bearer " + token
    headers["data-partition-id"] = data_partition_id
    headers["content-type"] = "application/json"
    if context["correlation_id"]:
        headers["correlation_id"] = context["correlation_id"]

    ent_response = auth_data.entitle_service.request_groups(headers=headers)
    groups = ent_response["groups"]
    logger.debug(
        f"response from entitlement service for {data_partition_id}: {groups} {auth_data.user_id} {auth_data.member_email}"
    )
    auth_data.authenticated = True

    # add info to auth_data
    try:
        # for val in response.json()['groups']:
        for val in groups:
            if isinstance(val, dict):
                auth_data.groups.append(val["name"])
        auth_data.member_email = ent_response["memberEmail"]
        auth_data.user_id = ent_response["desId"]
    except Exception:
        pass

    # print("Auth data groups:", auth_data.groups)

    # if user is not in USER_PERMISSION group and not and not in ADMIN group
    if (
        conf.USER_PERMISSION not in auth_data.groups
        and conf.ADMIN_PERMISSION not in auth_data.groups
    ):
        raise HTTPException(
            status_code=HTTP_401_UNAUTHORIZED,
            detail="User does not have the permission to call the API",
            headers={"WWW-Authenticate": "Bearer"},
        )
    logger.debug(f"Auth data OK {data_partition_id}")
    return auth_data


async def require_authorized_admin(
    request: Request, auth_data: Auth = Depends(require_authorized_user)
):
    if conf.ADMIN_PERMISSION not in auth_data.groups:
        raise HTTPException(
            status_code=HTTP_403_FORBIDDEN,
            detail="User does not have the permission to call the Admin API",
            headers={"WWW-Authenticate": "Bearer"},
        )
    return auth_data


async def require_authorized_ops(
    request: Request, auth_data: Auth = Depends(require_authorized_user)
):
    if conf.OPS_PERMISSION not in auth_data.groups:
        raise HTTPException(
            status_code=HTTP_403_FORBIDDEN,
            detail="User does not have the permission to call the Admin API",
            headers={"WWW-Authenticate": "Bearer"},
        )
    return auth_data
