import logging
import os

from typing import Tuple
from osdu_api.providers.types import FileLikeObject
from bundles.storage import BundleStorageClient

# from azure.storage.blob import BlobServiceClient, BlobClient, ContainerClient
from azure.storage.blob import BlobServiceClient
from azure.identity import DefaultAzureCredential
from azure.keyvault.secrets import SecretClient

logger = logging.getLogger(__name__)


class AzureBundleStorageClient(BundleStorageClient):
    def __init__(self) -> None:
        helper = Helper()
        self.client = BlobServiceClient.from_connection_string(
            helper.get_storage_connection_string()
        )
        self.account_name = os.environ["STORAGE_ACCOUNT"]
        self.container_name = os.environ["CONTAINER_NAME"]
        self.content_type = "application/x-gtar"

    def _get_uri(self, filename: str) -> str:
        return "https://{0}/{1}/{2}".format(
            self.account_name, self.container_name, filename
        )

    def download_file(
        self, filename: str, file: FileLikeObject
    ) -> Tuple[FileLikeObject, str]:
        try:
            uri = self._get_uri(filename)
            blob_client = self.client.get_blob_client(
                container=self.container_name, blob=filename
            )
            downloader = blob_client.download_blob()
            downloader.readinto(file)
            return file, uri
        except Exception as e:
            logger.error(f"Failed to download file from {uri}: {e}")

    def upload_file(self, name: str, file: FileLikeObject) -> str:
        try:
            uri = self._get_uri(name)
            blob_client = self.client.get_blob_client(
                container=self.container_name, blob=name
            )
            # not sure how to set content type
            blob_client.upload_blob(file.read(), overwrite=True, blob_type="BlockBlob")
            return uri
        except Exception as e:
            logger.error(f"Failed to upload file to {uri}: {e}")


class Helper(object):
    KEY_VAULT_URI = os.getenv("KEYVAULT_URI")

    def get_key_vault_secret(self, key: str):
        credential = DefaultAzureCredential()
        secret_client = SecretClient(
            vault_url=Helper.KEY_VAULT_URI, credential=credential
        )
        return secret_client.get_secret(key).value

    def get_storage_connection_string(self):
        return self.get_key_vault_secret("airflow-storage-connection")
