# test_integration_010.py
import pytest

# requires pytest_dependency to be installed
import unittest
import responses
from fastapi.testclient import TestClient
import requests
import re
import logging
import sys
import os
import base64
import json
from pathlib import Path

# so the directly in which you run pytest or unittest doesn't matter as much
sys.path.append(os.path.abspath(".."))
sys.path.append("tests")

# local
from main import app
from auth import auth
import conf
import testlib

BEARER = "Bearer "

# override dependency injection for authentication to entitlement service
from override_depends import (
    override_require_authorized_user,
    set_authorize_session,
    ADMIN_ONLY_SVC,
    USER_AND_ADMIN_SVC,
    USER_ONLY_SVC,
    OTHER_ONLY_SVC,
)

TEST_DATA_DIR = Path(__file__).resolve().parent / "data"

client = TestClient(app)


@pytest.mark.dependency(name="require_token")
def test_require_token(token):
    assert token is not None, "No token provided on command line"


def test_cloud_provider(cloud_provider):
    """Check CLOUD_PROVIDER"""
    assert cloud_provider is not None, "No CLOUD_PROVIDER env set"


@pytest.mark.opa_access
@pytest.mark.mock
def test_mock_health_api_endpoint_no_auth_opa_available(opa_access, cloud_provider):
    """Check Health API"""
    r = client.get(conf.SERVICE_BASE_PATH + "/health")
    # If OPA is available should get a 200
    # Some CSPs (like AWS) don't expose /health without auth so let's skip it in that case
    if r.status_code == 401 or r.status_code == 403:
        pytest.skip(f"Skipping mock /health unauth test for {cloud_provider}")
    assert r.status_code == 200, f"Expect /health endpoint to be available {r.text}"
    assert "Healthy" in r.text, "Expected Healthy in response text from /health"


def test_serviceurl_health_api_endpoint_no_auth_opa_available(
    service_url, cloud_provider
):
    """Check Health API"""
    url = service_url + conf.SERVICE_BASE_PATH + "/health"
    r = requests.get(url)
    # If OPA is available should get a 200
    # Some CSPs (like AWS) don't expose /health without auth so let's skip it in that case
    if r.status_code == 401 or r.status_code == 403:
        pytest.skip(f"Skipping service_url /health unauth test for {cloud_provider}")
    assert (
        r.status_code == 200
    ), f"Expect /health endpoint ({url}) to be available {r.text}"
    assert "Healthy" in r.text, "Expected Healthy in response text from /health"


@pytest.mark.dependency(name="require_token")
def test_serviceurl_health_api_endpoint_opa_available_auth_headers(
    token, data_partition, service_url
):
    """Check Health API with Auth headers"""
    url = service_url + conf.SERVICE_BASE_PATH + "/health"
    headers = {"Authorization": BEARER + token, "data-partition-id": data_partition}
    r = requests.get(url, headers=headers)
    # If OPA is available should get a 200
    # This should work in all cloud providers
    assert (
        r.status_code == 200
    ), f"Expect /health endpoint ({url}) to be available {r.status_code} {r.text}"
    assert "Healthy" in r.text, "Expected Healthy in response text from /health"


@pytest.mark.dependency(name="require_token")
def test_login(token, data_partition):
    """
    Check API endpoint for login
    This API is intended to be only used by AdminUI
    """
    if conf.ENABLE_ADMIN_UI == False:
        pytest.skip("Skipping adminui login test - ENABLE_ADMIN_UI")
    headers = {"Authorization": BEARER + token, "data-partition-id": data_partition}
    r = client.get(conf.SERVICE_BASE_PATH + "/login", headers=headers)
    if r.status_code == 401:
        print(f"problem with headers: {headers}")
    assert r.status_code == 200, f"Expected /login is available {r.text}"


@pytest.mark.dependency(name="require_token")
@pytest.mark.mock
def test_info_mock(token, data_partition):
    """
    Check info API endpoint
    """
    r = client.get(
        conf.SERVICE_BASE_PATH + "/info",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    js = json.dumps(r.json(), indent=4)
    print(f"{js}")
    assert (
        r.status_code == 200
    ), f"Expected /info is available text: {r.text}, status_code: {r.status_code} ENTITLEMENTS_GROUPS_API:{conf.ENTITLEMENTS_GROUPS_API}, OPA_URL: {conf.OPA_URL}, Environment variables: {os.environ}"


@pytest.mark.dependency(name="require_token")
def test_info_service_url(token, data_partition, service_url):
    """
    Check info API endpoint
    """
    url = service_url + conf.SERVICE_BASE_PATH + "/info"
    r = requests.get(
        url,
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    js = json.dumps(r.json(), indent=4)
    print(f"{js}")
    assert r.status_code == 200, f"Expected {url} is available {r.text}"


@pytest.mark.opa_access
@pytest.mark.dependency(depends=["require_token"])
def test_local_config(token, data_partition):
    """
    Check config API endpoint
    This endpoint should not be available in production environments and
    is only enabled when ENABLE_DEV_DIAGNOSTICS is set
    """
    if conf.ENABLE_DEV_DIAGNOSTICS == False:
        pytest.skip("Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")

    r = client.get(
        conf.SERVICE_BASE_PATH + "/config",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    print(f"text: {r.text}", file=sys.stderr)
    assert r.status_code == 200, f"Config Page is available {r.text}"


@pytest.mark.dependency(depends=["require_token"])
def test_config_service_url(token, data_partition, service_url):
    """
    Check config API endpoint
    This endpoint should not be available in production environments and
    is only enabled when ENABLE_DEV_DIAGNOSTICS is set
    """
    if conf.ENABLE_DEV_DIAGNOSTICS == False:
        pytest.skip("Skipping diagnostic test - ENABLE_DEV_DIAGNOSTICS")

    r = requests.get(
        service_url + conf.SERVICE_BASE_PATH + "/config",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    print(f"text: {r.text}", file=sys.stderr)
    assert r.status_code == 200, f"Config Page is available {r.text}"


def test_docs_api_endpoint(token, data_partition, service_url):
    """Check Docs endpoint"""
    response = requests.get(
        service_url + conf.SERVICE_BASE_PATH + "/docs",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    assert response.status_code == 200, "Expected /docs to be available"


def test_redocs_api_endpoint(token, data_partition, service_url):
    """Check ReDocs endpoint"""
    response = requests.get(
        service_url + conf.SERVICE_BASE_PATH + "/redoc",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    assert response.status_code == 200, "Expected Redoc to be available"


def test_openapi_json_endpoint(token, data_partition, service_url):
    """Check openapi json is available"""
    response = requests.get(
        service_url + conf.SERVICE_BASE_PATH + "/openapi.json",
        headers={"Authorization": BEARER + token, "data-partition-id": data_partition},
    )
    assert response.status_code == 200, "Expected OpenAPI.json to be available"
